@@ -3,6 +3,14 @@ from django.db import models
 from django.utils import simplejson
 
 
+RESULT_CHOICES = tuple((k, k) for k in (
+    'success',
+    'failed',
+    'skipped',
+    'deprecated',
+))
+
+
 class GzippedJSONField(models.TextField):
     """
     Slightly different from a JSONField in the sense that the default
@@ -64,6 +72,7 @@ class Build(models.Model):
     num_tests = models.PositiveIntegerField(default=0)
     total_duration = models.FloatField(default=0.0)
     data = GzippedJSONField(default={})
+    result = models.CharField(max_length=16, choices=RESULT_CHOICES, null=True)
 
     class Meta:
         unique_together = (('revision', 'datetime'),)
@@ -75,27 +84,36 @@ class Build(models.Model):
         self.project = self.revision.project
         super(Build, self).save(*args, **kwargs)
 
-    def get_previous_build(self):
+    def _get_temporal_sibling(self, datetime_filter, order_field, tag=None):
+        filter_args = {
+            'project': self.project,
+            datetime_filter: self.datetime
+        }
+        if tag:
+            filter_args["tags"] = tag
+
         try:
             return type(self).objects.filter(
-                project=self.project,
-                datetime__lt=self.datetime
+                **filter_args
             ).exclude(
                 id=self.id,
-            ).order_by('-datetime')[0]
+            ).order_by(order_field)[0]
         except IndexError:
             return None
 
-    def get_next_build(self):
-        try:
-            return type(self).objects.filter(
-                project=self.project,
-                datetime__gt=self.datetime
-            ).exclude(
-                id=self.id,
-            ).order_by('datetime')[0]
-        except IndexError:
-            return None
+    def get_previous_build(self, tag=None):
+        return self._get_temporal_sibling('datetime__lt', "-datetime", tag)
+
+    def get_next_build(self, tag=None):
+        return self._get_temporal_sibling('datetime__gt', "datetime", tag)
+
+
+class BuildTag(models.Model):
+    builds = models.ManyToManyField(Build, related_name='tags')
+    label = models.CharField(max_length=255)
+
+    def __unicode__(self):
+        return self.label
 
 
 class Test(models.Model):
@@ -111,6 +129,7 @@ class Test(models.Model):
     lower_duration = models.FloatField(default=0.0)
     upper90_duration = models.FloatField(default=0.0)
     data = GzippedJSONField(default={})
+    result = models.CharField(max_length=16, choices=RESULT_CHOICES, null=True)
 
     class Meta:
         unique_together = (('build', 'label'),)
